#--------------------------------------------------------------------------------
# 참조 모듈 목록.
#--------------------------------------------------------------------------------
from __future__ import annotations
from typing import Awaitable, Callable, Final, Generic, Iterable, Iterator, Optional, Sequence, Type, TypeVar, Tuple, Union
from typing import ItemsView, KeysView, ValuesView
from typing import IO, TextIO, BinaryIO
from typing import Any, List, Dict, Set
from typing import cast, overload
import builtins
from enum import Enum
import os
import re
import uuid
from xml.dom import minidom as Minidom
from xml.dom.minidom import Document as MinidomDocument
from xml.etree import ElementTree
from xml.etree.ElementTree import Element
from .custom import Custom


#--------------------------------------------------------------------------------
# 인스톨익스큐트시퀀스 요소.
#--------------------------------------------------------------------------------
class InstallExecuteSequence:
	#--------------------------------------------------------------------------------
	# 멤버 변수 목록.
	#--------------------------------------------------------------------------------
	FileKey: str
	Children: Optional[List[Custom]]


#--------------------------------------------------------------------------------
# 참조 모듈 목록.
#--------------------------------------------------------------------------------
from __future__ import annotations
from typing import Awaitable, Callable, Final, Generic, Iterable, Iterator, Optional, Sequence, Type, TypeVar, Tuple, Union
from typing import ItemsView, KeysView, ValuesView
from typing import IO, TextIO, BinaryIO
from typing import Any, List, Dict, Set
from typing import cast, overload
import builtins


#--------------------------------------------------------------------------------
# 패키지 포함 목록.
#--------------------------------------------------------------------------------
from ...commandmanager import CommandManager
from .core import IDatabase, IQuery, IRecord, ITable, Installer
from .customactiontype import CustomActionType
from .database import Database
from .databasescope import DatabaseScope
# from .native import *
from .proertytable import PropertyTable
from .query import Query
from .queryscope import QueryScope
from .record import Record
from .table import Table
from .tablefieldtype import TableFieldType


#--------------------------------------------------------------------------------
# 참조 모듈 목록.
#--------------------------------------------------------------------------------
from __future__ import annotations
from typing import Awaitable, Callable, Final, Generic, Iterable, Iterator, Optional, Sequence, Type, TypeVar, Tuple, Union
from typing import ItemsView, KeysView, ValuesView
from typing import IO, TextIO, BinaryIO
from typing import Any, List, Dict, Set
from typing import cast, overload
import builtins
from builtins import object as Object
import os
from xpl import SharedClass
from .directorynode import DirectoryNode
from .filenode import FileNode
from .inode import INode
from .nodetype import NodeType


#--------------------------------------------------------------------------------
# 노드 매니저.
#--------------------------------------------------------------------------------
class NodeManager(SharedClass):
	#--------------------------------------------------------------------------------
	# 멤버 변수 목록.
	#--------------------------------------------------------------------------------
	__nodes: dict[str, INode] = dict()


	#--------------------------------------------------------------------------------
	# 노드 목록 프로퍼티. (읽기 전용으로 수정해도 영향 없음)
	#--------------------------------------------------------------------------------
	@property
	def Nodes(self) -> List[INode]:
		values: ValuesView = self.__nodes.values()
		valueList = list(values)
		return valueList
	

	#--------------------------------------------------------------------------------
	# 생성됨.
	#--------------------------------------------------------------------------------
	def __init__(self) -> None:
		self.__nodes = dict()


	#--------------------------------------------------------------------------------
	# 노드 전체 파괴.
	#--------------------------------------------------------------------------------
	def DestroyAllNodes(self) -> None:
		if not self.__nodes:
			return	
		for node in self.Nodes:
			self.DestroyNode(node)
		self.__nodes.clear()


	#--------------------------------------------------------------------------------
	# 노드 추가.
	# - useUpdate: 이미 있을 경우 갱신처리 여부.
	#--------------------------------------------------------------------------------
	def AddNode(self, node: INode, useUpdate: bool = True) -> bool:
		if not node:
			return False
		if not useUpdate and self.ContainsNode(node.Value):
			return
		self.__nodes[node.Value] = node


	#--------------------------------------------------------------------------------
	# 노드 파괴. (함부로 삭제하면 캐싱이 꼬여서 곤란하므로 진짜 필요할 때 외에는 사용 금지)
	#--------------------------------------------------------------------------------
	def DestroyNode(self, target: Union[INode, str]) -> bool:
		if not target:
			return False
		targetNormalizedPath: str = NodeManager.GetNormalizedPath(target)
		if not targetNormalizedPath:
			return False	
		if targetNormalizedPath in self.__nodes:
			return False	
		target: INode = self.__nodes[targetNormalizedPath]
		target.OnDestroy()
		del self.__nodes[targetNormalizedPath]
		return True


	#--------------------------------------------------------------------------------
	# 노드 포함 여부 확인.
	#--------------------------------------------------------------------------------
	def ContainsNode(self, target: Union[INode, str]) -> bool:
		if not target:
			return False
		if not self.__nodes:
			return False
		targetNormalizedPath: str = NodeManager.GetNormalizedPath(target)
		if not targetNormalizedPath:
			return False
		if targetNormalizedPath not in self.__nodes:
			return False
		return True


	#--------------------------------------------------------------------------------
	# 노드 존재여부 및 노드 반환.
	#--------------------------------------------------------------------------------
	def TryGetNode(self, targetPath: str) -> Union[bool, Optional[INode]]:
		node: INode = self.GetNode(targetPath)
		if not node:
			return (False, None)
		return (True, node)


	#--------------------------------------------------------------------------------
	# 노드 반환.
	#--------------------------------------------------------------------------------
	def GetNode(self, target: Union[INode, str]) -> Optional[INode]:
		if not target:
			return None
		if not self.__nodes:
			return None
		targetNormalizedPath: str = NodeManager.GetNormalizedPath(target)
		if not targetNormalizedPath:
			return None
		if targetNormalizedPath not in self.__nodes:
			return None
		node: INode = self.__nodes[targetNormalizedPath]
		return node


	#--------------------------------------------------------------------------------
	# 노드 생성.
	# - 경로에서 디렉토리/파일의 실존 여부를 검사하지 않으므로 주의.
	#--------------------------------------------------------------------------------
	@staticmethod
	def CreateNode(targetPath: str) -> Optional[INode]:
		targetNormalizedPath = NodeManager.GetNormalizedPath(targetPath)
		if not targetNormalizedPath:
			return None
		nodeManager: NodeManager = NodeManager.GetSharedInstance()
		isContains, node = nodeManager.TryGetNode(targetNormalizedPath)
		if isContains:
			return node
		else:
			if NodeManager.IsDirectoryPath(targetNormalizedPath):
				node: INode = Object.__new__(DirectoryNode)
				nodeManager.AddNode(node)
				node.__init__(targetNormalizedPath)
				node.OnCreate(targetNormalizedPath)
				return node
			elif NodeManager.IsFilePath(targetNormalizedPath):
				node: INode = Object.__new__(FileNode)
				nodeManager.AddNode(node)
				node.__init__(targetNormalizedPath)
				node.OnCreate(targetNormalizedPath)
				return node
			else:
				return None


	#--------------------------------------------------------------------------------
	# 디렉토리 여부 반환.
	#--------------------------------------------------------------------------------
	@staticmethod
	def IsDirectoryPath(targetPath: str) -> bool:
		path, name, extension = NodeManager.GetPathAndNameAndExtension(targetPath)
		if extension:
			return False
		return True


	#--------------------------------------------------------------------------------
	# 파일 여부 반환.
	#--------------------------------------------------------------------------------
	@staticmethod
	def IsFilePath(targetPath: str) -> bool:
		path, name, extension = NodeManager.GetPathAndNameAndExtension(targetPath)
		if not extension:
			return False
		return True
	
	#--------------------------------------------------------------------------------
	# 디렉토리 존재 여부 반환.
	#--------------------------------------------------------------------------------
	@staticmethod
	def ExistsDirectory(targetDirectoryPath: str) -> bool:
		if not os.path.isdir(targetDirectoryPath):
			return False
		return True


	#--------------------------------------------------------------------------------
	# 파일 존재 여부 반환.
	#--------------------------------------------------------------------------------
	@staticmethod
	def ExistsFile(targetFilePath: str) -> bool:
		if not os.path.isfile(targetFilePath):
			return False
		return True


	#--------------------------------------------------------------------------------
	# 경로의 정규화된 경로 반환.
	#--------------------------------------------------------------------------------
	@staticmethod
	def GetNormalizedPath(target: Union[INode, str]) -> Optional[str]:
		if not target:
			return None
		if builtins.isinstance(target, INode):
			targetNormalizedPath = os.path.normpath(target.Value)
			return targetNormalizedPath
		elif builtins.isinstance(target, str):
			targetNormalizedPath = os.path.normpath(target)
			return targetNormalizedPath
		else:
			return None


	#--------------------------------------------------------------------------------
	# 노드/절대경로를 경로, 이름, 확장자로 변환하여 반환.
	#--------------------------------------------------------------------------------
	@staticmethod
	def GetPathAndNameAndExtension(targetPath: Union[INode, str]) -> tuple[str, str, str]:
		targetNormalizedPath: str = NodeManager.GetNormalizedPath(targetPath)
		if not targetNormalizedPath:
			return ("", "", "")
		path, fileName = os.path.split(targetNormalizedPath)
		name, extension = os.path.splitext(fileName)
		return (path, name, extension)


	#--------------------------------------------------------------------------------
	# 노드/경로 두개의 경로가 동일한지 여부 비교.
	#--------------------------------------------------------------------------------
	@staticmethod
	def Equals(leftTarget: Union[INode, str], rightTarget: Union[INode, str]) -> bool:
		if not leftTarget or not rightTarget:
			return False
		
		leftValue: str = NodeManager.GetNormalizedPath(leftTarget)
		rightValue: str = NodeManager.GetNormalizedPath(rightTarget)	
		if leftValue == rightValue:
			return True
		return False


#--------------------------------------------------------------------------------
# 참조 모듈 목록.
#--------------------------------------------------------------------------------
from __future__ import annotations
from typing import Awaitable, Callable, Final, Generic, Iterable, Iterator, Optional, Sequence, Type, TypeVar, Tuple, Union
from typing import ItemsView, KeysView, ValuesView
from typing import IO, TextIO, BinaryIO
from typing import Any, List, Dict, Set
from typing import cast, overload
import builtins
from enum import Enum


#--------------------------------------------------------------------------------
# Microsoft Installer Custom Action Type.
#--------------------------------------------------------------------------------
class CustomActionType(Enum):
	EXECUTE_EXE_PATH_PROVIDED = 1      # 실행 파일 호출. (경로 제공)
	EXECUTE_FILE_AFTER_INSTALL = 2     # 파일 설치 후 실행.
	CALL_DLL = 8                       # DLL 호출.
	EXECUTE_EXE_PROPERTY_BASED = 16    # EXE 호출. (Property 기반)
	EXECUTE_INSTALLED_FILE = 32        # 설치된 파일 실행. (경로 제공)
	EXECUTE_FILE_ID_BASED = 34         # 파일 실행. (File ID 기반)
	EXECUTE_PATH_PROPERTY_BASED = 50   # Property 기반 경로로 실행.
	RUN_INTERNAL_SCRIPT = 64           # 내부 코드 호출.
	CALL_DLL_PROPERTY_BASED = 128      # DLL 호출. (Property 기반)
	RUN_PROCESS_WITH_ADMIN = 256       # 보조 관리자 프로세스 실행.
	WAIT_PROCESS_AND_CONTINUE = 512    # 보조 관리자 프로세스 실행 후 종료.
	EXECUTE_OUTSIDE_UI = 1024          # UI 외부에서 실행.
	IGNORE_FAILURE = 2048              # 실패 시 무시.


#--------------------------------------------------------------------------------
# 참조 모듈 목록.
#--------------------------------------------------------------------------------
from __future__ import annotations
from typing import Awaitable, Callable, Final, Generic, Iterable, Iterator, Optional, Sequence, Type, TypeVar, Tuple, Union
from typing import ItemsView, KeysView, ValuesView
from typing import IO, TextIO, BinaryIO
from typing import Any, List, Dict, Set
from typing import cast, overload
import builtins
import re
import uuid
from xml.dom import minidom as Minidom
from xml.dom.minidom import Document as MinidomDocument
from xml.etree import ElementTree
from xml.etree.ElementTree import Element


#--------------------------------------------------------------------------------
# 전역 상수 목록.
#--------------------------------------------------------------------------------
FILE_READTEXT: str = "rt"
FILE_WRITETEXT: str = "wt"
UTF8: str = "utf-8"
EMPTY: str = ""
RE_REMOVE_NS0: str = "(ns0:|ns0|:ns0)"
LINEFEED: str = "\n"


#--------------------------------------------------------------------------------
# XML 기능.
#--------------------------------------------------------------------------------
class XML:
	#--------------------------------------------------------------------------------
	# ns0 제외하고 재저장.
	#--------------------------------------------------------------------------------
	@staticmethod
	def SaveWithoutNS0(xmlFilePath: str) -> None:
		# 파일 읽기.
		with builtins.open(xmlFilePath, mode = FILE_READTEXT, encoding = UTF8) as inputFile:
			content = inputFile.read()
			content = content.replace("ns0:", EMPTY)
			content = content.replace(":ns0", EMPTY)
			content = content.replace(":ns0", EMPTY)
		# 재저장.
		with builtins.open(xmlFilePath, mode = FILE_WRITETEXT, encoding = UTF8) as outputFile:
			outputFile.write(content)


#--------------------------------------------------------------------------------
# 참조 모듈 목록.
#--------------------------------------------------------------------------------
from __future__ import annotations
from typing import Awaitable, Callable, Final, Generic, Iterable, Iterator, Optional, Sequence, Type, TypeVar, Tuple, Union
from typing import ItemsView, KeysView, ValuesView
from typing import IO, TextIO, BinaryIO
from typing import Any, List, Dict, Set
from typing import cast, overload
import builtins
import os
# # import cx_Freeze # type: ignore
# # from cx_Freeze import Executable # type: ignore
# import win32com.client # type: ignore


# #--------------------------------------------------------------------------------
# # 전역 상수 목록.
# #--------------------------------------------------------------------------------
# NAME: str = "이름"
# VERSION: str = "버전"
# CREATOR: str = "공급업체"
# DESCRIPTION: str = "설명"
# BDIST_MSI: str = "bdist_msi"
# BUILD_EXE: str = "build_exe"
# CURRENTFILEPATH: str = os.path.abspath(__file__)
# ROOTPATH: str = os.path.dirname(CURRENTFILEPATH)


# #--------------------------------------------------------------------------------
# # 파일 진입점.
# #--------------------------------------------------------------------------------
# if __name__ == "__main__":
# 	msi_path = "example.msi"  # 수정할 MSI 파일 경로
# 	file_id = "UpdateBatFile"  # File 테이블의 File ID
# 	component_id = "MainComponent"  # Component 테이블의 Component ID
# 	directory_id = "TARGETDIR"  # Component 테이블의 Directory ID
# 	sequence = 1  # 설치 순서	
# 	msiFilePath: str = os.path.join(ROOTPATH, msi_path)

# 	installer = win32com.client.Dispatch("WindowsInstaller.Installer")
# 	database = installer.OpenDatabase(msiFilePath, 3)

# 	tables_query = "SELECT * FROM `_Tables` WHERE Name = 'Component'"
# 	view = database.OpenView(tables_query)
# 	view.Execute(None)
# 	record = view.Fetch()
# 	if record:
# 		print("Component 테이블이 존재합니다.")
# 	else:
# 		print("Component 테이블이 없습니다.")

# 	tables_query = "SELECT * FROM `_Tables` WHERE Name = 'File'"
# 	view = database.OpenView(tables_query)
# 	view.Execute(None)
# 	record = view.Fetch()
# 	if record:
# 		print("File 테이블이 존재합니다.")
# 	else:
# 		print("File 테이블이 없습니다.")
# 		create_table_query = """
# 			CREATE TABLE File (
# 				File TEXT PRIMARY KEY,
# 				Component_ TEXT NOT NULL,
# 				FileName TEXT NOT NULL,
# 				FileSize INTEGER,
# 				Version TEXT,
# 				Language TEXT,
# 				Attributes INTEGER,
# 				Sequence INTEGER
# 			)
# 		"""
# 		view = database.OpenView(create_table_query)
# 		view.Execute(None)
# 		view.Close()

# 	try:
# 		sourceFilePath: str = os.path.join(ROOTPATH, "res", "msi", "update.bat")
# 		sourceFileSize = os.path.getsize(sourceFilePath)
# 		view = database.OpenView("INSERT INTO File (File, Component_, FileName, FileSize, Version, Language, Attributes, Sequence) VALUES (?, ?, ?, ?, ?, ?, ?, ?)")
# 		record = installer.CreateRecord(8)
# 		record.SetString(1, file_id)  # File ID
# 		record.SetString(2, component_id)  # Component ID
# 		record.SetString(3, "update.bat")  # File Name
# 		record.SetInteger(4, sourceFileSize)  # File Size
# 		record.SetString(5, None)  # Version
# 		record.SetString(6, None)  # Language
# 		record.SetInteger(7, 0)  # Attributes
# 		record.SetInteger(8, sequence)  # Sequence	
# 		view.Execute(record)
# 		view.Close()
# 		view = database.OpenView("INSERT INTO Component (Component, ComponentId, Directory_, Attributes, Condition, KeyPath) VALUES (?, ?, ?, ?, ?, ?)")
# 		record = installer.CreateRecord(6)
# 		record.SetString(1, component_id)  # Component ID
# 		record.SetString(2, "{12345678-1234-1234-1234-123456789ABC}")  # Component GUID
# 		record.SetString(3, directory_id)  # Directory ID
# 		record.SetInteger(4, 0)  # Attributes
# 		record.SetString(5, None)  # Condition
# 		record.SetString(6, file_id)  # KeyPath
# 		view.Execute(record)	
# 		view.Close()

# 		with open(sourceFilePath, "rb") as file:
# 			bytes = file.read()

# 		database.Streams(file_id).Write(bytes)
# 		database.Commit()
# 	except Exception as exception:
# 		builtins.print(exception)
#	# finally:
#	# 	database.Close()


#--------------------------------------------------------------------------------
# 참조 모듈 목록.
#--------------------------------------------------------------------------------
from __future__ import annotations
from typing import Awaitable, Callable, Final, Generic, Iterable, Iterator, Optional, Sequence, Type, TypeVar, Tuple, Union
from typing import ItemsView, KeysView, ValuesView
from typing import IO, TextIO, BinaryIO
from typing import Any, List, Dict, Set
from typing import cast, overload
import builtins


#--------------------------------------------------------------------------------
# 패키지 포함 목록.
#--------------------------------------------------------------------------------
from .directorynode import DirectoryNode
from .filenode import FileNode
from .inode import INode
from .nodemanager import NodeManager
from .nodetype import NodeType


#--------------------------------------------------------------------------------
# 참조 모듈 목록.
#--------------------------------------------------------------------------------
from __future__ import annotations
from typing import Awaitable, Callable, Final, Generic, Iterable, Iterator, Optional, Sequence, Type, TypeVar, Tuple, Union
from typing import ItemsView, KeysView, ValuesView
from typing import IO, TextIO, BinaryIO
from typing import Any, List, Dict, Set
from typing import cast, overload
import builtins
import os
from .core import IRecord


#--------------------------------------------------------------------------------
# Microsoft Installer Database의 레코드.
#--------------------------------------------------------------------------------
class Record(IRecord):
	#--------------------------------------------------------------------------------
	# 필드 갯수 반환.
	#--------------------------------------------------------------------------------
	def GetFieldCount(self) -> int:
		rawdata = self.GetRawData()
		return rawdata.GetFieldCount()


	#--------------------------------------------------------------------------------
	# 값 반환.
	#--------------------------------------------------------------------------------
	def GetFieldValueAsInteger(self, fieldIndex: int) -> int:
		if fieldIndex < 0 or fieldIndex >= self.GetFieldCount():
			raise IndexError(f"invalid field index: {fieldIndex}")
		rawdata = self.GetRawData()
		return rawdata.GetInteger(fieldIndex + 1)


	#--------------------------------------------------------------------------------
	# 값 반환.
	#--------------------------------------------------------------------------------
	def GetFieldValueAsString(self, fieldIndex: int) -> str:
		if fieldIndex < 0 or fieldIndex >= self.GetFieldCount():
			raise IndexError(f"invalid field index: {fieldIndex}")
		rawdata = self.GetRawData()
		return rawdata.GetString(fieldIndex + 1)


	#--------------------------------------------------------------------------------
	# 값 설정.
	#--------------------------------------------------------------------------------
	def GetFieldValue(self, fieldIndex: int) -> Union[int, str, None]:
		if fieldIndex < 0 or fieldIndex >= self.GetFieldCount():
			raise IndexError(f"invalid field index: {fieldIndex}")
		try:
			return self.GetFieldValueAsInteger(fieldIndex)
		except Exception as exception:
			try:
				return self.GetFieldValueAsString(fieldIndex)
			except Exception as exception:
				raise


	#--------------------------------------------------------------------------------
	# 값 설정.
	#--------------------------------------------------------------------------------
	def SetFieldValueAsInteger(self, fieldIndex: int, value: int) -> None:
		if fieldIndex < 0 or fieldIndex >= self.GetFieldCount():
			raise IndexError(f"invalid field index: {fieldIndex}")
		elif value == None:
			rawdata = self.GetRawData()
			rawdata.SetString(fieldIndex + 1, 0)
		elif not builtins.isinstance(value, int):
			raise ValueError(f"filePath is Not Integer.")
		else:
			rawdata = self.GetRawData()
			rawdata.SetInteger(fieldIndex + 1, value)


	#--------------------------------------------------------------------------------
	# 값 설정.
	#--------------------------------------------------------------------------------
	def SetFieldValueAsString(self, fieldIndex: int, value: str) -> None:
		if fieldIndex < 0 or fieldIndex >= self.GetFieldCount():
			raise IndexError(f"invalid field index: {fieldIndex}")
		elif value == None:
			rawdata = self.GetRawData()
			rawdata.SetString(fieldIndex + 1, "")
		elif builtins.isinstance(value, str):
			rawdata = self.GetRawData()
			rawdata.SetString(fieldIndex + 1, value)
		else:
			raise ValueError(f"{value} is Not String.")


	#--------------------------------------------------------------------------------
	# 값 설정.
	#--------------------------------------------------------------------------------
	def SetFieldValueAsStream(self, fieldIndex: int, filePath: str) -> None:
		if fieldIndex < 0 or fieldIndex >= self.GetFieldCount():
			raise IndexError(f"invalid field index: {fieldIndex}")
		elif not builtins.isinstance(filePath, str):
			raise ValueError(f"not String.")
		elif not os.path.exists(filePath):
			raise FileNotFoundError(f"not exist file: {filePath}")
		rawdata = self.GetRawData()
		rawdata.SetStream(fieldIndex + 1, filePath)


	#--------------------------------------------------------------------------------
	# 값 설정.
	#--------------------------------------------------------------------------------
	def SetFieldValue(self, fieldIndex: int, value: Union[int, str, None]) -> None:
		if fieldIndex < 0 or fieldIndex >= self.GetFieldCount():
			raise IndexError(f"invalid field index: {fieldIndex}")
		elif value == None:
			self.SetFieldValueAsString(fieldIndex, None)
		elif builtins.isinstance(value, int):
			self.SetFieldValueAsInteger(fieldIndex, value)
		elif builtins.isinstance(value, str):
			if os.path.exists(value):
				self.SetFieldValueAsStream(fieldIndex, value)
			else:
				self.SetFieldValueAsString(fieldIndex, value)
		else:
			raise ValueError(f"connot be a field value: {value}")


	#--------------------------------------------------------------------------------
	# 값 설정.
	#--------------------------------------------------------------------------------
	def SetFieldValues(self, fieldValues: Tuple) -> None:
		fieldIndex: int = 0
		for fieldValue in fieldValues:
			self.SetFieldValue(fieldIndex, fieldValue)
			fieldIndex += 1


#--------------------------------------------------------------------------------
# 참조 모듈 목록.
#--------------------------------------------------------------------------------
from __future__ import annotations
from typing import Awaitable, Callable, Final, Generic, Iterable, Iterator, Optional, Sequence, Type, TypeVar, Tuple, Union
from typing import ItemsView, KeysView, ValuesView
from typing import IO, TextIO, BinaryIO
from typing import Any, List, Dict, Set
from typing import cast, overload
import builtins
from enum import Enum
import os
import re
import uuid
from xml.dom import minidom as Minidom
from xml.dom.minidom import Document as MinidomDocument
from xml.etree import ElementTree
from xml.etree.ElementTree import Element
from .condition import Condition
from .customaction import CustomAction
from .feature import Feature
from .guidattribute import GuidAttribute
from .idattribute import IdAttribute
from .installexecutesequence import InstallExecuteSequence
from .mediatemplate import MediaTemplate
from .package import Package
from .property import Property
from .ui import UI
from .wixvariable import WixVariable


#--------------------------------------------------------------------------------
# 프로덕트 요소.
#--------------------------------------------------------------------------------
class Product:
	#--------------------------------------------------------------------------------
	# 멤버 변수 목록.
	#--------------------------------------------------------------------------------
	Id: IdAttribute
	Name: str
	Manufacturer: str
	UpgradeCode: GuidAttribute
	Version: str
	Language: str
	Children: Optional[List[Condition, CustomAction, Feature, InstallExecuteSequence, MediaTemplate, Package, Property, UI, WixVariable]]